/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.api.web.gwt.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.i18n.client.Dictionary;

public abstract class AbstractResourceTab implements EntryPoint {
  public static final String HTML_ROOT_ID = "sourceviewers";

  ResourceDictionary resourceDictionary;
  Panel container = null;
  SourcePanel sourcePanel = null;

  public void onModuleLoad() {
    this.resourceDictionary = new ResourceDictionary();
    exportJavascript();
  }

  protected abstract String getGwtId();

  protected abstract void exportJavascript();

  private native void refreshSourceViewer(String pluginKey) /*-{
      $wnd.config['sourceviewer_plugin_key'] = pluginKey;
      $wnd.callGwtViewer('org.sonar.plugins.core.resourceviewer.GwtResourceViewer');
    }-*/;


  public void loadSource() {
    loadContainer();

    String resourceKey = resourceDictionary.getSourceKey();
    if (resourceKey != null) {
      if (sourcePanel == null || !resourceKey.equals(sourcePanel.getResourceKey())) {
        if (sourcePanel != null) {
          sourcePanel.removeFromParent();
          sourcePanel = null;
        }

        sourcePanel = new SourcePanel(resourceKey, getDecorator());
        container.add(sourcePanel);

      }
      container.setVisible(true);
    }
  }

  protected abstract SourceDecorator getDecorator();

  protected Widget loadContainer() {
    if (container == null) {
      RootPanel rootPanel = RootPanel.get(HTML_ROOT_ID);
      if (rootPanel != null) {
        container = new FlowPanel();
        container.setStyleName("gwt-ResourceTab");
        String id = "tab-" + getGwtId();
        container.getElement().setId(id);
        registerTab(id);

        final TabBar tabs = new TabBar();
        tabs.setWidth("100%");
        container.add(tabs);

        final Dictionary tabsDictionnary = resourceDictionary.getSourceViewers();
        int tabIndex = 0;
        for (String tabTitle : tabsDictionnary.keySet()) {
          tabs.addTab(tabTitle);
          if (tabsDictionnary.get(tabTitle).equals(getGwtId())) {
            tabs.selectTab(tabIndex);
          }
          tabIndex++;
        }
        tabs.addTabListener(new TabListener() {
          public boolean onBeforeTabSelected(SourcesTabEvents sourcesTabEvents, int i) {
            String gwtId = (String) tabsDictionnary.values().toArray()[i];
            refreshSourceViewer(gwtId);
            return false;
          }

          public void onTabSelected(SourcesTabEvents sourcesTabEvents, int i) {
          }
        });

        rootPanel.add(container);
      }
    }
    return container;
  }

  private static native void registerTab(Object tabId) /*-{
    $wnd.registeredTabs.push(tabId);
  }-*/;

}
