#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Api::SourcesController < Api::RestController

  def index
    pid = params[:project_id]
    conditions='enabled=:enabled AND islast=:islast AND projects.scope=:scope AND status=:status '
    if pid.to_i>0
      conditions+=' AND projects.id=:pid'
      pid=pid.to_i
    else
      conditions+=' AND kee=:pid'
    end

    project = Project.find(:first, :include => {:snapshots => :source},
      :conditions => [conditions,
        {:pid => pid, :enabled => true, :islast => true, :scope => Project::SCOPE_CLASS, :status => Snapshot::STATUS_PROCESSED}] )

    if project
      source = project.snapshots[0].source
      source = SnapshotSource.new if !source
      rest_render(source)
    else
      rest_status_ko("No sources for [" + params[:project_id] + "]", 404)
    end
  end

  private

  def rest_render(source)
    respond_to do |format|
      format.json{ render :json => rest_to_json(source) }
      format.xml { render :xml => rest_to_xml(source) }
      format.text { render :text => rest_to_text(source) }
    end
  end

  def rest_to_json(source)
    json = [source.to_hash_json()].to_json
    json = params['callback'] + '(' + json + ');' if params['callback']
    json
  end

  def rest_to_xml(source)
    xml = Builder::XmlMarkup.new(:indent => 0)
    xml.instruct!
    xml << source.to_xml
  end

  def rest_to_text(source)
    if source and source.data
      source.data
    else
      ""
    end
  end

end