#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
require 'java'
include_class "org.sonar.core.batch.monitoring.MonitorImpl"

class ProjectController < ApplicationController
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [  :set_links, :add_review, :update_default_treemap_metrics ], :redirect_to => { :action => :index }
  verify :method => :delete, :only => [ :delete, :delete_review ], :redirect_to => { :action => :index }

  require_role Role::ADMIN, :for => [ :administration, :delete, :set_links, :delete_review, :add_review, :edit_review, :update_default_treemap_metrics]

  before_filter :init_treemap_opts, :only => [:index, :modules_treemap]

  SECTION=Navigation::SECTION_PROJECT

  TREEMAP_SIZE_ON_DASHBOARD_PAGE = 200

  def index
    if params[:sid]
      @snapshot = Snapshot.find(:first, :conditions => {:id => params[:sid].to_i, :status => Snapshot::STATUS_PROCESSED })
      unless @snapshot
        redirect_to_default
        return
      end
      @project = @snapshot.project
    else
      @project = Project.by_key(params[:id])
      unless @project
        redirect_to_default
        return
      end
      @snapshot = @project.last_snapshot
    end

    @snapshot_ids = []
    @snapshot_ids = params[:sid].split(',').collect {|s| s.to_i} if params[:sid]

    @default_color_metric = configuration.getProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_COLOR_METRIC) || Sonar::Treemap.default_color_metric
    @default_size_metric = configuration.getProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_SIZE_METRIC) || Sonar::Treemap.default_size_metric
    init_treemap

  rescue ActiveRecord::RecordNotFound
    redirect_to_default
  end

  def modules_treemap
    # params[:id] is the snapshot id, not the project id
    snapshots = Snapshot.find(:all, :conditions => ['projects.enabled=1 and parent_snapshot_id=%s', params[:id].to_i], :include => 'project')
    measures_by_snapshot = get_measures_by_snapshot(snapshots)
    treemap = Sonar::Treemap.new(measures_by_snapshot, params)
    color_metric_direction = Sonar::Treemap.get_color_metric_direction(params)
    render(:update) do |page|
      page.replace_html 'treemap', treemap.generate_html
      page.replace_html 'treemap_gradient', :partial => 'shared/treemap_gradient', :locals => { :direction => color_metric_direction }
      page.replace_html 'treemap_set_default', :partial => 'project/treemap_set_default',
        :locals => { :controller => 'project', :size_name => params[:size_name], :color_name => params[:color_name],
          :project_key => params[:project_key]}
    end
  end

  def update_default_treemap_metrics
    configuration.setProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_COLOR_METRIC, params[:color_name])
    configuration.setProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_SIZE_METRIC, params[:size_name])
    redirect_to :action => 'index', :id => params[:project_key]
  end

  def show_reviews
    @snapshot = Snapshot.find(params[:sid].to_i)
    render :partial => 'dashboard_reviews'
  end

  def edit_review
    @snapshot = Snapshot.find(params[:sid].to_i)
    @review = ProjectMeasure.new(:measure_date => @snapshot.created_at, :value => 0)
    @review_types = Metric.review_types
    render :partial => 'dashboard_edit_review'
  end

  def add_review
    @snapshot=Snapshot.find(params[:sid].to_i)
    measure = ProjectMeasure.new(params[:review])
    measure.project = @snapshot.project

    if measure.measure_date <= @snapshot.created_at
      if measure.metric.val_type==Metric::VALUE_TYPE_STRING
        measure.text_value=params['review']['value']
        measure.value=0
      end
      begin
        measure.save!
        java_facade.registerAsyncMeasure(measure.id.to_i)
        if not params[:url].blank?
          measure.measure_parameters.create(:param_key => 'url', :text_value => params[:url])
        end
        if not params[:summary].blank?
          measure.measure_parameters.create(:param_key => 'summary', :text_value => params[:summary])
        end
        flash[:notice] = 'Measure added'
      rescue
        flash[:error] = measure.errors.full_messages.join("<br/>")
      end

    else
      flash[:error] = "The date should not be after #{@snapshot.created_at.strftime('%Y-%m-%d')}"
    end

    if request.xhr?
      render :update do |page|
        page.redirect_to :action => 'index', :id => measure.project_id
      end
    else
      redirect_to :action => 'index', :id => measure.project_id
    end
  end

  def delete_review
    measure = ProjectMeasure.find(params[:id].to_i)
    if measure and measure.review?
      java_facade.deleteAsyncMeasure(measure.id.to_i)
      redirect_to :action => 'index', :id => measure.project_id
    else
      redirect_to_default
    end
  end

  def delete
    if params[:id]
      pid=params[:id].to_i
      Project.update_all(['enabled=?', false], ['id=? or root_id=?', pid, pid])
    end
    redirect_to_default
  end

  def set_links
    project = Project.find( params[:project_id].to_i )
    project.links.clear

    params.each_pair do |key, value|
      if (key.starts_with?('href_') and value.size>2)
        project.links.create(:href => value, :link_type => key[5..-1])

      elsif (key.starts_with?('custom_href_') and value.size>2)
        id = key[12..-1]
        if (params['custom_name_' + id])
          project.links.create(:href => value, :name => params['custom_name_' + id])
        end
      end
    end
    project.save!

    flash[:notice] = 'Links updated.'
    redirect_to :action => 'administration', :id => project.id
  end

  def administration
    @project=Project.find(params[:id].to_i)
    @snapshot=@project.last_snapshot
  end


  def events
    @categories=EventCategory.categories(configuration, true)
    @snapshot=Snapshot.find(params[:id])
    @category=params[:category]
    conditions={:resource_id => @snapshot.project_id}
    conditions[:category]=@category unless @category.blank?
    @events=Event.find(:all, :conditions => conditions, :order => 'event_date desc')
    render :action => 'events', :layout => ! request.xhr?
  end


  protected

  def init_treemap_opts
    params[:width] = TREEMAP_SIZE_ON_DASHBOARD_PAGE
    params[:height] = TREEMAP_SIZE_ON_DASHBOARD_PAGE
    params[:display_branch_name] = false
    # the parameter _id_  will be replaced by Sonar::HtmlOutput
    params[:link_url] = url_for :controller => 'project', :action => 'index', :id => '_id_', :only_path => true
  end

  def get_treemap_metrics
    color_name = params[:color_name] ? params[:color_name] : @default_color_metric
    size_name = params[:size_name] ? params[:size_name] : @default_size_metric
    color_metric = Metric.by_name(color_name)
    size_metric = Metric.by_name(size_name)
    [color_metric, size_metric]
  end

  def init_treemap
    params[:size_name] = @default_size_metric
    params[:color_name] = @default_color_metric
    measures_by_snapshot = get_measures_by_snapshot(@snapshot.children)
    @treemap = Sonar::Treemap.new(measures_by_snapshot, params)
    @color_metric_direction = Sonar::Treemap.get_color_metric_direction(params)
  end

  def refresh_configure
    render :update do |page|
      page.replace_html("rule_id_#{@rule.id}", :partial => 'rule', :locals => { :rule => @rule })
    end
  end

  def redirect_to_default
    redirect_to :controller => 'projects', :action => 'index'
  end

  def get_measures_by_snapshot(snapshots)
    if snapshots.size>0
      measures = project_measures(snapshots, get_treemap_metrics)
      measures_by_snapshot(snapshots, measures)
    else
      nil
    end
  end

  def measures_by_snapshot(snapshots, measures)
    snapshot_by_id = {}
    snapshots.each {|s| snapshot_by_id[s.id]=s}
    hash={}
    measures.each do |m|
      hash[snapshot_by_id[m.snapshot_id]] ||= []
      hash[snapshot_by_id[m.snapshot_id]] << m
    end
    hash
  end

  def project_measures(snapshots, metrics)
    sids = snapshots.collect{|s| s.id}
    if sids and sids.size>0
      measures= ProjectMeasure.find(:all,
        :conditions => {
          'snapshot_id' => sids,
          'metric_id' => metrics.collect{|metric| metric.id},
          'rule_id' => nil})
    end
    measures || []
  end

end