#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
require 'java'
include_class "org.sonar.core.batch.monitoring.MonitorImpl"

class ProjectsController < ApplicationController

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => :update_default_treemap_metrics, :redirect_to => { :action => :index }
  require_role Role::ADMIN, :only => :update_default_treemap_metrics

  before_filter :init_treemap_opts, :only => [:index, :projects_treemap]

  TREEMAP_SIZE_ON_LIST_PAGE = 280
  SECTION = Navigation::SECTION_HOME

  def index
    @default_color_metric = configuration.getProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_COLOR_METRIC) || Sonar::Treemap.default_color_metric
    @default_size_metric = configuration.getProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_SIZE_METRIC) || Sonar::Treemap.default_size_metric

    @dashboard_configuration = Sonar::DashboardConfiguration.new(configuration)

    @snapshots = Snapshot.last_enabled_projects
    @columns = @dashboard_configuration.displayed_columns
    metrics = @dashboard_configuration.homepage_metrics(get_treemap_metrics)

    measures = project_measures(@snapshots, metrics)
    @measures_by_snapshot = measures_by_snapshot(@snapshots, measures)

    init_treemap if @dashboard_configuration.is_treemap_enabled? and @snapshots.size>1

    monitor = Java::OrgSonarCore::Application.getInstance().getContainer().getComponent(Java::OrgSonarCoreBatchMonitoring::MonitorImpl.java_class)
    @started = monitor.getStarted()
    @scheduled = monitor.getScheduled()

    if logged_in?
      @sidebar='projects/sidebar'
    end
  end

  def projects_treemap
    params[:display_branch_name] = false
    measures_by_snapshot = get_measures_by_snapshot(Snapshot.last_enabled_projects)
    treemap = Sonar::Treemap.new(measures_by_snapshot, params)
    color_metric_direction = Sonar::Treemap.get_color_metric_direction(params)
    render(:update) do |page|
      page.replace_html 'treemap', treemap.generate_html
      page.replace_html 'treemap_gradient', :partial => 'shared/treemap_gradient',
        :locals => { :direction => color_metric_direction }
      page.replace_html 'treemap_set_default', :partial => 'projects/treemap_set_default',
        :locals => { :controller => 'projects', :size_name => params[:size_name], :color_name => params[:color_name] }
    end
  end

  def update_default_treemap_metrics
    configuration.setProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_COLOR_METRIC, params[:color_name])
    configuration.setProperty(Sonar::Treemap::CONFIGURATION_DEFAULT_SIZE_METRIC, params[:size_name])
    redirect_to :action => 'index'
  end

  protected

  def init_treemap_opts
    params[:width] = TREEMAP_SIZE_ON_LIST_PAGE
    params[:height] = TREEMAP_SIZE_ON_LIST_PAGE
    # the parameter _id_  will be replaced by Sonar::HtmlOutput
    params[:link_url] = url_for :controller => 'project', :action => 'index', :id => '_id_', :only_path => true
    params[:display_branch_name] = true
  end

  def get_treemap_metrics
    color_name = params[:color_name] ? params[:color_name] : @default_color_metric
    size_name = params[:size_name] ? params[:size_name] : @default_size_metric
    color_metric = Metric.by_name(color_name)
    size_metric = Metric.by_name(size_name)
    [color_metric, size_metric]
  end

  def init_treemap   
    params[:size_name] = @default_size_metric
    params[:color_name] = @default_color_metric
    @treemap = Sonar::Treemap.new(@measures_by_snapshot, params)
    @color_metric_direction = Sonar::Treemap.get_color_metric_direction(params)
  end

  def refresh_configure
    render :update do |page|
      page.replace_html("rule_id_#{@rule.id}", :partial => 'rule', :locals => { :rule => @rule })
    end
  end

  def get_measures_by_snapshot(snapshots)
    if snapshots.size>0
      measures = project_measures(snapshots, get_treemap_metrics)
      measures_by_snapshot(snapshots, measures)
    else
      nil
    end
  end

  def measures_by_snapshot(snapshots, measures)
    snapshot_by_id = {}
    snapshots.each {|s| snapshot_by_id[s.id]=s}
    hash={}
    measures.each do |m|
      hash[snapshot_by_id[m.snapshot_id]] ||= []
      hash[snapshot_by_id[m.snapshot_id]] << m
    end
    hash
  end

  def project_measures(snapshots, metrics)
    sids = snapshots.collect{|s| s.id}
    if sids and sids.size>0
      mids = metrics.collect{|metric| metric.id}
      measures=ProjectMeasure.find(:all, :conditions => {
          'snapshot_id' => sids,
          'metric_id' => mids,
          'rule_id' => nil})
      async_measures=AsyncMeasureSnapshot.search(sids, mids)
      measures + async_measures
    else
      []
    end
  end

end
