#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class BreakdownColumn
  attr_reader :selected_snapshots
  
  def initialize(parent_snapshot_ids, metric_key, selected_snapshot_ids=[], rules_category_id=nil, rule_id=nil)
    @snapshots=Snapshot.find(:all, :conditions => {:parent_snapshot_id => parent_snapshot_ids}, :include => 'project')
    @metric=load_metric(metric_key)
        
    rule_conditions = {}
    if rule_id
      rule_conditions[:rule_id]=rule_id
    elsif rules_category_id
      rule_conditions[:rules_category_id]=rules_category_id
      rule_conditions[:rule_id]=nil
    else
      rule_conditions[:rules_category_id]=nil
      rule_conditions[:rule_id]=nil
    end
    @measures=[]
    if @snapshots.size>0
      # this is a DERBY bug. If empty array, then SQL is SELECT ... WHERE SNAPSHOT_ID IN (NULL) ==> error
      @measures=ProjectMeasure.find(:all, 
        :conditions => {
          :snapshot_id => @snapshots.map {|s| s.id}, 
          :metric_id => @metric.id}.merge(rule_conditions),
          :order => 'value')
    end
    
    if @metric.direction<0
      @measures.reverse!
    end
    
    #optimization : snapshots and projects are already loaded
    snapshot_by_id=array_to_hash_by_id(@snapshots)
    @measures.each do |measure|
      measure.snapshot=snapshot_by_id[measure.snapshot_id]
    end
    
    @selected_snapshots=@snapshots.select{|s| selected_snapshot_ids.include?(s.id)}
  end
  
  def display?
    @snapshots.size>0
  end

  def measures(include_zero=true)
    if include_zero
      @measures
    else
      @measures.select{|m| m.value>0}
    end
  end
  
  def display_next_column?
    @selected_snapshots.size==1
  end
  
  def snapshots_for_next_column
    @selected_snapshots.size>0 ? @selected_snapshots : @snapshots
  end
  
  def selected_sid?(sid)
      for snapshot in @selected_snapshots
        if snapshot.id==sid
          return true
        end
      end
      false
  end
  
  private
  def load_metric(metric_key)
    if metric_key==Metric::DUPLICATED_LINES_RATIO
      Metric::by_key(Metric::PMD_DUPLICATED_LINES)
    else
      Metric::by_key(metric_key)
    end
  end

  def array_to_hash_by_id(array)
    hash={}
    array.each do |s|
      hash[s.id]=s
    end
    hash
  end
end