#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with {library}; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class RuleFailure < ActiveRecord::Base

  has_many :rule_failure_params
  belongs_to :rule

  LEVEL_INFO = 0
  LEVEL_WARNING = 1
  LEVEL_ERROR = 2

  def string_failure_level
    case failure_level
    when LEVEL_WARNING
      'warning'
    when LEVEL_ERROR
      'error'
    when LEVEL_INFO
      'info'
    end
  end

  def error?
    failure_level==LEVEL_ERROR
  end

  def warning?
    failure_level==LEVEL_WARNING
  end

  def line
    @line ||=
      begin
        lines=rule_failure_params.select{|p| p.param_key=='line'}
        if lines and !lines.empty?
          lines.first.value.to_i
        else
          nil
        end
      end
  end

  def to_hash_json
    json = {}
    json['rule'] = rule.name
    json['message'] = message
    json['line'] = line if line
    json['level'] = string_failure_level
    json
  end

  def to_xml
    xml = Builder::XmlMarkup.new
    xml.violation do
      xml.rule(rule.name)
      xml.message(message)
      xml.line(line) if line
      xml.level(string_failure_level)
    end
  end

end
