 #
 # Sonar, entreprise quality control tool.
 # Copyright (C) 2007-2008 Hortis-GRC SA
 # mailto:be_agile HAT hortis DOT ch
 #
 # Sonar is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 3 of the License, or (at your option) any later version.
 #
 # Sonar is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with {library}; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 #
require 'java'
include_class 'org.slf4j.LoggerFactory'
include_class 'org.slf4j.MarkerFactory'

class Slf4rLogger < Logger
  
  def add(severity, message = nil, progname = nil, &block)
    severity ||= UNKNOWN
    progname ||= @progname
    progname ||= "default"
    marker = MarkerFactory::getMarker( progname )
    if message.nil?
      if block_given?
        message = yield
      else
        message = progname
      end
    end
    logger.debug(marker, message) if severity == Logger::DEBUG
    logger.info(marker, message) if severity == Logger::INFO
    logger.warn(marker, message) if severity == Logger::WARN
    logger.error(marker, message) if severity == Logger::ERROR or severity == Logger::FATAL
    logger.trace(marker, message) if severity == Logger::UNKNOWN
    true
  end
  
  def <<(msg)
    add(Logger::UNKNOWN, msg, nil)
  end
    
  private
  
  def logger
	  if @logger.nil?
		  logger_name = "rails." + @progname.to_s if ( @progname )
		  logger_name = "rails.default" if ( @progname.nil? )
		  @logger = LoggerFactory::getLogger( logger_name );
      @level = nil
      @level ||= Logger::ERROR if @logger.isErrorEnabled()
      @level ||= Logger::WARN if @logger.isWarnEnabled()
      @level ||= Logger::INFO if @logger.isInfoEnabled()
      @level ||= Logger::DEBUG if @logger.isDebugEnabled() or @logger.isTraceEnabled()
	  end
    @logger 	
  end

end